# ------------------------------------
# Copyright (c) Microsoft Corporation.
# Licensed under the MIT License.
# ------------------------------------

from ._client import CertificateClient

__all__ = ["CertificateClient"]
