# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

try:
    from ._models_py3 import ErrorResponse
    from ._models_py3 import LiveTokenResponse
    from ._models_py3 import OperationInfo
    from ._models_py3 import OperationLive
    from ._models_py3 import OperationsListResult
except (SyntaxError, ImportError):
    from ._models import ErrorResponse  # type: ignore
    from ._models import LiveTokenResponse  # type: ignore
    from ._models import OperationInfo  # type: ignore
    from ._models import OperationLive  # type: ignore
    from ._models import OperationsListResult  # type: ignore

__all__ = [
    'ErrorResponse',
    'LiveTokenResponse',
    'OperationInfo',
    'OperationLive',
    'OperationsListResult',
]
