# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

try:
    from ._models_py3 import ARMErrorResponseBody
    from ._models_py3 import ArmErrorResponse
    from ._models_py3 import ConfigData
    from ._models_py3 import ConfigurationListResult
    from ._models_py3 import DigestConfig
    from ._models_py3 import MetadataEntity
    from ._models_py3 import MetadataEntityListResult
    from ._models_py3 import MetadataSupportedValueDetail
    from ._models_py3 import OperationDisplayInfo
    from ._models_py3 import OperationEntity
    from ._models_py3 import OperationEntityListResult
    from ._models_py3 import Resource
    from ._models_py3 import ResourceMetadata
    from ._models_py3 import ResourceRecommendationBase
    from ._models_py3 import ResourceRecommendationBaseListResult
    from ._models_py3 import ShortDescription
    from ._models_py3 import SuppressionContract
    from ._models_py3 import SuppressionContractListResult
except (SyntaxError, ImportError):
    from ._models import ARMErrorResponseBody  # type: ignore
    from ._models import ArmErrorResponse  # type: ignore
    from ._models import ConfigData  # type: ignore
    from ._models import ConfigurationListResult  # type: ignore
    from ._models import DigestConfig  # type: ignore
    from ._models import MetadataEntity  # type: ignore
    from ._models import MetadataEntityListResult  # type: ignore
    from ._models import MetadataSupportedValueDetail  # type: ignore
    from ._models import OperationDisplayInfo  # type: ignore
    from ._models import OperationEntity  # type: ignore
    from ._models import OperationEntityListResult  # type: ignore
    from ._models import Resource  # type: ignore
    from ._models import ResourceMetadata  # type: ignore
    from ._models import ResourceRecommendationBase  # type: ignore
    from ._models import ResourceRecommendationBaseListResult  # type: ignore
    from ._models import ShortDescription  # type: ignore
    from ._models import SuppressionContract  # type: ignore
    from ._models import SuppressionContractListResult  # type: ignore

from ._advisor_management_client_enums import (
    Category,
    ConfigurationName,
    CpuThreshold,
    DigestConfigState,
    Impact,
    Risk,
    Scenario,
)

__all__ = [
    'ARMErrorResponseBody',
    'ArmErrorResponse',
    'ConfigData',
    'ConfigurationListResult',
    'DigestConfig',
    'MetadataEntity',
    'MetadataEntityListResult',
    'MetadataSupportedValueDetail',
    'OperationDisplayInfo',
    'OperationEntity',
    'OperationEntityListResult',
    'Resource',
    'ResourceMetadata',
    'ResourceRecommendationBase',
    'ResourceRecommendationBaseListResult',
    'ShortDescription',
    'SuppressionContract',
    'SuppressionContractListResult',
    'Category',
    'ConfigurationName',
    'CpuThreshold',
    'DigestConfigState',
    'Impact',
    'Risk',
    'Scenario',
]
