# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

from enum import Enum, EnumMeta
from six import with_metaclass

class _CaseInsensitiveEnumMeta(EnumMeta):
    def __getitem__(self, name):
        return super().__getitem__(name.upper())

    def __getattr__(cls, name):
        """Return the enum member matching `name`
        We use __getattr__ instead of descriptors or inserting into the enum
        class' __dict__ in order to support `name` and `value` being both
        properties for enum members (which live in the class' __dict__) and
        enum members themselves.
        """
        try:
            return cls._member_map_[name.upper()]
        except KeyError:
            raise AttributeError(name)


class CreatedByType(with_metaclass(_CaseInsensitiveEnumMeta, str, Enum)):
    """The type of identity that created the resource.
    """

    USER = "User"
    APPLICATION = "Application"
    MANAGED_IDENTITY = "ManagedIdentity"
    KEY = "Key"

class KeyType(with_metaclass(_CaseInsensitiveEnumMeta, str, Enum)):
    """Whether the operation refers to the primary or secondary key.
    """

    PRIMARY = "primary"
    SECONDARY = "secondary"

class Kind(with_metaclass(_CaseInsensitiveEnumMeta, str, Enum)):
    """The Kind of the Maps Account.
    """

    GEN1 = "Gen1"
    GEN2 = "Gen2"

class Name(with_metaclass(_CaseInsensitiveEnumMeta, str, Enum)):
    """The name of the SKU, in standard format (such as S0).
    """

    S0 = "S0"
    S1 = "S1"
    G2 = "G2"
