# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

from enum import Enum
from azure.core import CaseInsensitiveEnumMeta


class CreatedByType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The type of identity that created the resource."""

    USER = "User"
    APPLICATION = "Application"
    MANAGED_IDENTITY = "ManagedIdentity"
    KEY = "Key"


class DataBoundary(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The data boundary definition."""

    NOT_DEFINED = "NotDefined"
    GLOBAL = "Global"
    EU = "EU"
    GLOBAL_ENUM = "Global"


class DefaultName(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """DefaultName."""

    DEFAULT = "default"


class ProvisioningState(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Denotes the state of provisioning."""

    ACCEPTED = "Accepted"
    RUNNING = "Running"
    CREATING = "Creating"
    CANCELED = "Canceled"
    FAILED = "Failed"
    SUCCEEDED = "Succeeded"
    UPDATING = "Updating"
