# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

try:
    from ._models_py3 import Creator
    from ._models_py3 import CreatorList
    from ._models_py3 import CreatorProperties
    from ._models_py3 import CreatorUpdateParameters
    from ._models_py3 import Dimension
    from ._models_py3 import ErrorAdditionalInfo
    from ._models_py3 import ErrorDetail
    from ._models_py3 import ErrorResponse
    from ._models_py3 import MapsAccount
    from ._models_py3 import MapsAccountKeys
    from ._models_py3 import MapsAccountProperties
    from ._models_py3 import MapsAccountUpdateParameters
    from ._models_py3 import MapsAccounts
    from ._models_py3 import MapsKeySpecification
    from ._models_py3 import MapsOperations
    from ._models_py3 import MetricSpecification
    from ._models_py3 import OperationDetail
    from ._models_py3 import OperationDisplay
    from ._models_py3 import Resource
    from ._models_py3 import ServiceSpecification
    from ._models_py3 import Sku
    from ._models_py3 import SystemData
    from ._models_py3 import TrackedResource
except (SyntaxError, ImportError):
    from ._models import Creator  # type: ignore
    from ._models import CreatorList  # type: ignore
    from ._models import CreatorProperties  # type: ignore
    from ._models import CreatorUpdateParameters  # type: ignore
    from ._models import Dimension  # type: ignore
    from ._models import ErrorAdditionalInfo  # type: ignore
    from ._models import ErrorDetail  # type: ignore
    from ._models import ErrorResponse  # type: ignore
    from ._models import MapsAccount  # type: ignore
    from ._models import MapsAccountKeys  # type: ignore
    from ._models import MapsAccountProperties  # type: ignore
    from ._models import MapsAccountUpdateParameters  # type: ignore
    from ._models import MapsAccounts  # type: ignore
    from ._models import MapsKeySpecification  # type: ignore
    from ._models import MapsOperations  # type: ignore
    from ._models import MetricSpecification  # type: ignore
    from ._models import OperationDetail  # type: ignore
    from ._models import OperationDisplay  # type: ignore
    from ._models import Resource  # type: ignore
    from ._models import ServiceSpecification  # type: ignore
    from ._models import Sku  # type: ignore
    from ._models import SystemData  # type: ignore
    from ._models import TrackedResource  # type: ignore

from ._azure_maps_management_client_enums import (
    CreatedByType,
    KeyType,
    Kind,
    Name,
)

__all__ = [
    'Creator',
    'CreatorList',
    'CreatorProperties',
    'CreatorUpdateParameters',
    'Dimension',
    'ErrorAdditionalInfo',
    'ErrorDetail',
    'ErrorResponse',
    'MapsAccount',
    'MapsAccountKeys',
    'MapsAccountProperties',
    'MapsAccountUpdateParameters',
    'MapsAccounts',
    'MapsKeySpecification',
    'MapsOperations',
    'MetricSpecification',
    'OperationDetail',
    'OperationDisplay',
    'Resource',
    'ServiceSpecification',
    'Sku',
    'SystemData',
    'TrackedResource',
    'CreatedByType',
    'KeyType',
    'Kind',
    'Name',
]
