# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) Python Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

from ._models import AffinityInfo
from ._models import AuthenticationTokenSettings
from ._models import AutoScaleRun
from ._models import AutoScaleRunError
from ._models import AutoUserSpecification
from ._models import AutomaticOsUpgradePolicy
from ._models import AzureBlobFileSystemConfiguration
from ._models import AzureFileShareConfiguration
from ._models import BatchAccountListSupportedImagesResult
from ._models import BatchApplication
from ._models import BatchApplicationListResult
from ._models import BatchApplicationPackageReference
from ._models import BatchAutoPoolSpecification
from ._models import BatchError
from ._models import BatchErrorDetail
from ._models import BatchErrorMessage
from ._models import BatchJob
from ._models import BatchJobConstraints
from ._models import BatchJobCreateContent
from ._models import BatchJobDisableContent
from ._models import BatchJobExecutionInfo
from ._models import BatchJobListResult
from ._models import BatchJobManagerTask
from ._models import BatchJobNetworkConfiguration
from ._models import BatchJobPreparationAndReleaseTaskStatus
from ._models import BatchJobPreparationAndReleaseTaskStatusListResult
from ._models import BatchJobPreparationTask
from ._models import BatchJobPreparationTaskExecutionInfo
from ._models import BatchJobReleaseTask
from ._models import BatchJobReleaseTaskExecutionInfo
from ._models import BatchJobSchedule
from ._models import BatchJobScheduleConfiguration
from ._models import BatchJobScheduleCreateContent
from ._models import BatchJobScheduleExecutionInfo
from ._models import BatchJobScheduleListResult
from ._models import BatchJobScheduleStatistics
from ._models import BatchJobScheduleUpdateContent
from ._models import BatchJobSchedulingError
from ._models import BatchJobSpecification
from ._models import BatchJobStatistics
from ._models import BatchJobTerminateContent
from ._models import BatchJobUpdateContent
from ._models import BatchNode
from ._models import BatchNodeAgentInfo
from ._models import BatchNodeCounts
from ._models import BatchNodeDisableSchedulingContent
from ._models import BatchNodeEndpointConfiguration
from ._models import BatchNodeError
from ._models import BatchNodeFile
from ._models import BatchNodeFileListResult
from ._models import BatchNodeIdentityReference
from ._models import BatchNodeInfo
from ._models import BatchNodeListResult
from ._models import BatchNodePlacementConfiguration
from ._models import BatchNodeRebootContent
from ._models import BatchNodeRemoteLoginSettings
from ._models import BatchNodeRemoveContent
from ._models import BatchNodeUserCreateContent
from ._models import BatchNodeUserUpdateContent
from ._models import BatchNodeVMExtension
from ._models import BatchNodeVMExtensionListResult
from ._models import BatchPool
from ._models import BatchPoolCreateContent
from ._models import BatchPoolEnableAutoScaleContent
from ._models import BatchPoolEndpointConfiguration
from ._models import BatchPoolEvaluateAutoScaleContent
from ._models import BatchPoolIdentity
from ._models import BatchPoolInfo
from ._models import BatchPoolListResult
from ._models import BatchPoolListUsageMetricsResult
from ._models import BatchPoolNodeCounts
from ._models import BatchPoolNodeCountsListResult
from ._models import BatchPoolReplaceContent
from ._models import BatchPoolResizeContent
from ._models import BatchPoolResourceStatistics
from ._models import BatchPoolSpecification
from ._models import BatchPoolStatistics
from ._models import BatchPoolUpdateContent
from ._models import BatchPoolUsageMetrics
from ._models import BatchPoolUsageStatistics
from ._models import BatchStartTask
from ._models import BatchStartTaskInfo
from ._models import BatchSubtask
from ._models import BatchSupportedImage
from ._models import BatchTask
from ._models import BatchTaskAddCollectionResult
from ._models import BatchTaskAddResult
from ._models import BatchTaskConstraints
from ._models import BatchTaskContainerExecutionInfo
from ._models import BatchTaskContainerSettings
from ._models import BatchTaskCounts
from ._models import BatchTaskCountsResult
from ._models import BatchTaskCreateContent
from ._models import BatchTaskDependencies
from ._models import BatchTaskExecutionInfo
from ._models import BatchTaskFailureInfo
from ._models import BatchTaskGroup
from ._models import BatchTaskIdRange
from ._models import BatchTaskInfo
from ._models import BatchTaskListResult
from ._models import BatchTaskListSubtasksResult
from ._models import BatchTaskSchedulingPolicy
from ._models import BatchTaskSlotCounts
from ._models import BatchTaskStatistics
from ._models import CifsMountConfiguration
from ._models import ContainerConfiguration
from ._models import ContainerRegistryReference
from ._models import DataDisk
from ._models import DiffDiskSettings
from ._models import DiskEncryptionConfiguration
from ._models import EnvironmentSetting
from ._models import ExitCodeMapping
from ._models import ExitCodeRangeMapping
from ._models import ExitConditions
from ._models import ExitOptions
from ._models import FileProperties
from ._models import HttpHeader
from ._models import ImageReference
from ._models import InboundEndpoint
from ._models import InboundNatPool
from ._models import InstanceViewStatus
from ._models import LinuxUserConfiguration
from ._models import ManagedDisk
from ._models import MetadataItem
from ._models import MountConfiguration
from ._models import MultiInstanceSettings
from ._models import NameValuePair
from ._models import NetworkConfiguration
from ._models import NetworkSecurityGroupRule
from ._models import NfsMountConfiguration
from ._models import OSDisk
from ._models import OutputFile
from ._models import OutputFileBlobContainerDestination
from ._models import OutputFileDestination
from ._models import OutputFileUploadConfig
from ._models import PublicIpAddressConfiguration
from ._models import RecentBatchJob
from ._models import ResizeError
from ._models import ResourceFile
from ._models import RollingUpgradePolicy
from ._models import SecurityProfile
from ._models import ServiceArtifactReference
from ._models import UefiSettings
from ._models import UpgradePolicy
from ._models import UploadBatchServiceLogsContent
from ._models import UploadBatchServiceLogsResult
from ._models import UserAccount
from ._models import UserAssignedIdentity
from ._models import UserIdentity
from ._models import VMExtension
from ._models import VMExtensionInstanceView
from ._models import VirtualMachineConfiguration
from ._models import VirtualMachineInfo
from ._models import WindowsConfiguration
from ._models import WindowsUserConfiguration

from ._enums import AccessScope
from ._enums import AllocationState
from ._enums import AutoUserScope
from ._enums import BatchJobAction
from ._enums import BatchJobPreparationTaskState
from ._enums import BatchJobReleaseTaskState
from ._enums import BatchJobScheduleState
from ._enums import BatchJobState
from ._enums import BatchNodeCommunicationMode
from ._enums import BatchNodeDeallocationOption
from ._enums import BatchNodeDisableSchedulingOption
from ._enums import BatchNodeFillType
from ._enums import BatchNodePlacementPolicyType
from ._enums import BatchNodeRebootOption
from ._enums import BatchNodeState
from ._enums import BatchPoolIdentityType
from ._enums import BatchPoolLifetimeOption
from ._enums import BatchPoolState
from ._enums import BatchStartTaskState
from ._enums import BatchSubtaskState
from ._enums import BatchTaskAddStatus
from ._enums import BatchTaskExecutionResult
from ._enums import BatchTaskState
from ._enums import CachingType
from ._enums import ContainerType
from ._enums import ContainerWorkingDirectory
from ._enums import DependencyAction
from ._enums import DiffDiskPlacement
from ._enums import DisableBatchJobOption
from ._enums import DiskEncryptionTarget
from ._enums import DynamicVNetAssignmentScope
from ._enums import ElevationLevel
from ._enums import ErrorCategory
from ._enums import ImageVerificationType
from ._enums import InboundEndpointProtocol
from ._enums import IpAddressProvisioningType
from ._enums import LoginMode
from ._enums import NetworkSecurityGroupRuleAccess
from ._enums import OSType
from ._enums import OnAllBatchTasksComplete
from ._enums import OnBatchTaskFailure
from ._enums import OutputFileUploadCondition
from ._enums import SchedulingState
from ._enums import SecurityTypes
from ._enums import StatusLevelTypes
from ._enums import StorageAccountType
from ._enums import UpgradeMode
from ._patch import __all__ as _patch_all
from ._patch import *  # pylint: disable=unused-wildcard-import
from ._patch import patch_sdk as _patch_sdk

__all__ = [
    "AffinityInfo",
    "AuthenticationTokenSettings",
    "AutoScaleRun",
    "AutoScaleRunError",
    "AutoUserSpecification",
    "AutomaticOsUpgradePolicy",
    "AzureBlobFileSystemConfiguration",
    "AzureFileShareConfiguration",
    "BatchAccountListSupportedImagesResult",
    "BatchApplication",
    "BatchApplicationListResult",
    "BatchApplicationPackageReference",
    "BatchAutoPoolSpecification",
    "BatchError",
    "BatchErrorDetail",
    "BatchErrorMessage",
    "BatchJob",
    "BatchJobConstraints",
    "BatchJobCreateContent",
    "BatchJobDisableContent",
    "BatchJobExecutionInfo",
    "BatchJobListResult",
    "BatchJobManagerTask",
    "BatchJobNetworkConfiguration",
    "BatchJobPreparationAndReleaseTaskStatus",
    "BatchJobPreparationAndReleaseTaskStatusListResult",
    "BatchJobPreparationTask",
    "BatchJobPreparationTaskExecutionInfo",
    "BatchJobReleaseTask",
    "BatchJobReleaseTaskExecutionInfo",
    "BatchJobSchedule",
    "BatchJobScheduleConfiguration",
    "BatchJobScheduleCreateContent",
    "BatchJobScheduleExecutionInfo",
    "BatchJobScheduleListResult",
    "BatchJobScheduleStatistics",
    "BatchJobScheduleUpdateContent",
    "BatchJobSchedulingError",
    "BatchJobSpecification",
    "BatchJobStatistics",
    "BatchJobTerminateContent",
    "BatchJobUpdateContent",
    "BatchNode",
    "BatchNodeAgentInfo",
    "BatchNodeCounts",
    "BatchNodeDisableSchedulingContent",
    "BatchNodeEndpointConfiguration",
    "BatchNodeError",
    "BatchNodeFile",
    "BatchNodeFileListResult",
    "BatchNodeIdentityReference",
    "BatchNodeInfo",
    "BatchNodeListResult",
    "BatchNodePlacementConfiguration",
    "BatchNodeRebootContent",
    "BatchNodeRemoteLoginSettings",
    "BatchNodeRemoveContent",
    "BatchNodeUserCreateContent",
    "BatchNodeUserUpdateContent",
    "BatchNodeVMExtension",
    "BatchNodeVMExtensionListResult",
    "BatchPool",
    "BatchPoolCreateContent",
    "BatchPoolEnableAutoScaleContent",
    "BatchPoolEndpointConfiguration",
    "BatchPoolEvaluateAutoScaleContent",
    "BatchPoolIdentity",
    "BatchPoolInfo",
    "BatchPoolListResult",
    "BatchPoolListUsageMetricsResult",
    "BatchPoolNodeCounts",
    "BatchPoolNodeCountsListResult",
    "BatchPoolReplaceContent",
    "BatchPoolResizeContent",
    "BatchPoolResourceStatistics",
    "BatchPoolSpecification",
    "BatchPoolStatistics",
    "BatchPoolUpdateContent",
    "BatchPoolUsageMetrics",
    "BatchPoolUsageStatistics",
    "BatchStartTask",
    "BatchStartTaskInfo",
    "BatchSubtask",
    "BatchSupportedImage",
    "BatchTask",
    "BatchTaskAddCollectionResult",
    "BatchTaskAddResult",
    "BatchTaskConstraints",
    "BatchTaskContainerExecutionInfo",
    "BatchTaskContainerSettings",
    "BatchTaskCounts",
    "BatchTaskCountsResult",
    "BatchTaskCreateContent",
    "BatchTaskDependencies",
    "BatchTaskExecutionInfo",
    "BatchTaskFailureInfo",
    "BatchTaskGroup",
    "BatchTaskIdRange",
    "BatchTaskInfo",
    "BatchTaskListResult",
    "BatchTaskListSubtasksResult",
    "BatchTaskSchedulingPolicy",
    "BatchTaskSlotCounts",
    "BatchTaskStatistics",
    "CifsMountConfiguration",
    "ContainerConfiguration",
    "ContainerRegistryReference",
    "DataDisk",
    "DiffDiskSettings",
    "DiskEncryptionConfiguration",
    "EnvironmentSetting",
    "ExitCodeMapping",
    "ExitCodeRangeMapping",
    "ExitConditions",
    "ExitOptions",
    "FileProperties",
    "HttpHeader",
    "ImageReference",
    "InboundEndpoint",
    "InboundNatPool",
    "InstanceViewStatus",
    "LinuxUserConfiguration",
    "ManagedDisk",
    "MetadataItem",
    "MountConfiguration",
    "MultiInstanceSettings",
    "NameValuePair",
    "NetworkConfiguration",
    "NetworkSecurityGroupRule",
    "NfsMountConfiguration",
    "OSDisk",
    "OutputFile",
    "OutputFileBlobContainerDestination",
    "OutputFileDestination",
    "OutputFileUploadConfig",
    "PublicIpAddressConfiguration",
    "RecentBatchJob",
    "ResizeError",
    "ResourceFile",
    "RollingUpgradePolicy",
    "SecurityProfile",
    "ServiceArtifactReference",
    "UefiSettings",
    "UpgradePolicy",
    "UploadBatchServiceLogsContent",
    "UploadBatchServiceLogsResult",
    "UserAccount",
    "UserAssignedIdentity",
    "UserIdentity",
    "VMExtension",
    "VMExtensionInstanceView",
    "VirtualMachineConfiguration",
    "VirtualMachineInfo",
    "WindowsConfiguration",
    "WindowsUserConfiguration",
    "AccessScope",
    "AllocationState",
    "AutoUserScope",
    "BatchJobAction",
    "BatchJobPreparationTaskState",
    "BatchJobReleaseTaskState",
    "BatchJobScheduleState",
    "BatchJobState",
    "BatchNodeCommunicationMode",
    "BatchNodeDeallocationOption",
    "BatchNodeDisableSchedulingOption",
    "BatchNodeFillType",
    "BatchNodePlacementPolicyType",
    "BatchNodeRebootOption",
    "BatchNodeState",
    "BatchPoolIdentityType",
    "BatchPoolLifetimeOption",
    "BatchPoolState",
    "BatchStartTaskState",
    "BatchSubtaskState",
    "BatchTaskAddStatus",
    "BatchTaskExecutionResult",
    "BatchTaskState",
    "CachingType",
    "ContainerType",
    "ContainerWorkingDirectory",
    "DependencyAction",
    "DiffDiskPlacement",
    "DisableBatchJobOption",
    "DiskEncryptionTarget",
    "DynamicVNetAssignmentScope",
    "ElevationLevel",
    "ErrorCategory",
    "ImageVerificationType",
    "InboundEndpointProtocol",
    "IpAddressProvisioningType",
    "LoginMode",
    "NetworkSecurityGroupRuleAccess",
    "OSType",
    "OnAllBatchTasksComplete",
    "OnBatchTaskFailure",
    "OutputFileUploadCondition",
    "SchedulingState",
    "SecurityTypes",
    "StatusLevelTypes",
    "StorageAccountType",
    "UpgradeMode",
]
__all__.extend([p for p in _patch_all if p not in __all__])
_patch_sdk()
