# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
# pylint: disable=wrong-import-position

from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from ._patch import *  # pylint: disable=unused-wildcard-import

from ._operations import Operations  # type: ignore
from ._redis_operations import RedisOperations  # type: ignore
from ._firewall_rules_operations import FirewallRulesOperations  # type: ignore
from ._patch_schedules_operations import PatchSchedulesOperations  # type: ignore
from ._linked_server_operations import LinkedServerOperations  # type: ignore
from ._private_endpoint_connections_operations import PrivateEndpointConnectionsOperations  # type: ignore
from ._private_link_resources_operations import PrivateLinkResourcesOperations  # type: ignore
from ._async_operation_status_operations import AsyncOperationStatusOperations  # type: ignore
from ._access_policy_operations import AccessPolicyOperations  # type: ignore
from ._access_policy_assignment_operations import AccessPolicyAssignmentOperations  # type: ignore

from ._patch import __all__ as _patch_all
from ._patch import *
from ._patch import patch_sdk as _patch_sdk

__all__ = [
    "Operations",
    "RedisOperations",
    "FirewallRulesOperations",
    "PatchSchedulesOperations",
    "LinkedServerOperations",
    "PrivateEndpointConnectionsOperations",
    "PrivateLinkResourcesOperations",
    "AsyncOperationStatusOperations",
    "AccessPolicyOperations",
    "AccessPolicyAssignmentOperations",
]
__all__.extend([p for p in _patch_all if p not in __all__])  # pyright: ignore
_patch_sdk()
